# coding: utf-8

"""
    Mozart platform API

    API for interacting with the Mozart platform.  # noqa: E501

    The version of the OpenAPI document: 0.2.0
    Contact: support@bang-olufsen.dk
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import mozart_api
from mozart_api.models.sound_feature_set import SoundFeatureSet  # noqa: E501
from mozart_api.rest import ApiException


class TestSoundFeatureSet(unittest.TestCase):
    """SoundFeatureSet unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test SoundFeatureSet
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # model = mozart_api.models.sound_feature_set.SoundFeatureSet()  # noqa: E501
        if include_optional:
            return SoundFeatureSet(
                ambience=null,
                balance=null,
                bass=null,
                bass_management=null,
                compression=null,
                directivity=null,
                fader=null,
                room_compensation=null,
                spatial_envelopment=null,
                spatial_height=null,
                spatial_processing=null,
                spatial_surround=null,
                spatial_width=null,
                speech_enhance=null,
                tone_touch_x=null,
                tone_touch_y=null,
                treble=null,
            )
        else:
            return SoundFeatureSet()

    def testSoundFeatureSet(self):
        """Test SoundFeatureSet"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
